// PageHandler.js
// BEGIN COPYRIGHT
// *************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-C96
// (C) Copyright IBM Corporation 2012, 2012. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
// *************************************************************************
// END COPYRIGHT

dojo.provide("com.ibm.bpm.social.portal.PageHandler");

dojo.require("dojo.cookie");
dojo.require("dojo.hash");
dojo.require("com.ibm.bpm.social.util.common.Log");
dojo.require("com.ibm.bpm.social.widgets.task.config.FilterConfig");
dojo.require("com.ibm.bpm.social.widgets.task.config.ViewConfig");
dojo.require("com.ibm.bpm.social.util.common.PageSettings");
dojo.require("com.ibm.bpm.social.portal.DashboardDropdown");

dojo.requireLocalization("com.ibm.bpm.social", "bpmSocial");

dojo
  .declare(
    "com.ibm.bpm.social.portal.PageHandler",
    null,
    {

      /**
       * PageHandler : a class handling setting and getting the default start page for the work tab
       * 
       * <p>
       * Mapping of pages to string:
       * 
       * <table width="100%" >
       * 
       * <tr>
       * <td><b> Page </b></td>
       * <td><b> Stored string </b></td>
       * </tr>
       * <tr>
       * <td> My Tasks </td>
       * <td> /tasks </td>
       * </tr>
       * <tr>
       * <td> Dashboards tab </td>
       * <td> /dashboards?dashboard=PATH_TO_DASHBOARD </td>
       * </tr>
       * <tr>
       * <td> My Tasks - Open Tasks </td>
       * <td> /tasks/open?searchFilter=<i>URL_encoded_search_string</i> </td>
       * </tr>
       * <tr>
       * <td> My Tasks - Completed Tasks </td>
       * <td> /tasks/completed?searchFilter=<i>URL_encoded_search_string</i> </td>
       * </tr>
       * <tr>
       * <td> </td>
       * <td> </td>
       * </tr>
       * <tr>
       * <td> Saved Search </td>
       * <td> /tasks/queries?query=<i>URL_encoded_name_of_saved_search</i> </td>
       * </tr>
       * <tr>
       * <td> Saved Search with search string </td>
       * <td> /tasks/queries?query=<i>URL_encoded_name_of_saved_search</i>&amp;searchFilter=<i>URL_encoded_search_string</i>
       * </td>
       * </tr>
       * <tr>
       * <td> Sorted Saved Search with search string </td>
       * <td> /tasks/queries<b>?query</b>=<i>URL_encoded_name_of_saved_search</i><b>&amp;searchFilter</b>=<i>URL_encoded_search_string</i><b>&amp;sortAttributes</b>=<i>sortAttr1,sortAttr2,..</i><b>&amp;sortOrder</b>=<i>order1,order2,...</i>
       * </td>
       * </tr>
       * </table>
       * 
       */


      _storedStartPageSettings: null,
      _currentPageSettings: null,

      _startPageButton: null, // DOM node of the icon/button we are maintaining

      // DOM connections tracker
      _connects: [],

      _activePageWidget: null,
      _workPageWidget: null,
      _dashboardPageWidget: null,


      /**
       * Constructor
       * 
       * @type {void}
       */
      constructor: function constructor_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        this._nlsResources = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmSocial");

        // Create the dashboard dropdown initially to avoid issues with 
        // updates before dropdown is first clicked/created
        this.createDashboardDropdown();
        this.dashboardDropdown.hide();

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Return a copy of PageSettings of this instance. <br>
       * Since we return a copy, we can protect our PageSettings object.
       * 
       * @type {void}
       */
      getStoredPageSettings: function getStoredPageSettings_PageHandler(){
        com.ibm.bpm.social.util.common.Log.log(3);
        return new com.ibm.bpm.social.util.common.PageSettings(this._storedStartPageSettings);
      },


      /**
       * Cleanup handler <br>
       * Note: since this is not a widget, needs to called manually
       * 
       * @type {void}
       */
      destroy: function destroy_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        while (this._connects.length > 0) {
          var c = this._connects.pop();
          try {
            dojo.disconnect(c);
          }
          catch (e) {
            // we are not really interested in errors while cleanup  
          }
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Read the users profile setting "Portal default page" and parse it into the members
       * 
       * @returns {com.ibm.bpm.social.util.common.PageSettings}
       */
      readFromUserProfile: function readFromUserProfile_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var userProfile = com.ibm.bpm.socialportal.GlobalAssets.getUserProfile();
        var startPageSettings = new com.ibm.bpm.social.util.common.PageSettings();
        var userProfileString;

        if (userProfile && userProfile.userPreferences &&
          userProfile.userPreferences[com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME]) {
          userProfileString = userProfile.userPreferences[com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME];
          com.ibm.bpm.social.util.common.Log.log(2, "userProfile string is " + userProfileString);

          var newStartPageSettings = startPageSettings.parseFromString(userProfileString, true);
          if (!newStartPageSettings.isValid) {
            // String is invalid
            // the original startPageSettings object is unmodified and has default values
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is invalid, using default string: " +
              startPageSettings.getCurrentProfileString());
          }
          else {
            // the string is valid and the values are mixed into the original PageSettings object
            com.ibm.bpm.social.util.common.Log.log(2, "profile string is valid, using now string: " +
              startPageSettings.getCurrentProfileString());
          }
        }
        else {
          // if the start page is not set, show My Tasks
          com.ibm.bpm.social.util.common.Log.log(2, "user attribute not set, using default string: " +
            startPageSettings.getCurrentProfileString());
        }


        if (com.ibm.bpm.social.util.common.Log.logLevel > 1) {
          com.ibm.bpm.social.util.common.Log.logObject(2, {
            Tab: startPageSettings.tab,
            Filter: startPageSettings.filterConfigId,
            Saved_Search: startPageSettings.savedSearchName,
            Search_Filter: startPageSettings.searchFilterString,
            Sort_Attributes: startPageSettings.sortAttributes,
            dashboardURL: startPageSettings.dashboardURL
          }, "Got following settings from user profile: ");
        }

        // update the _storedStartPageSettings to the newest settings, but use a separate copy
        this._storedStartPageSettings = new com.ibm.bpm.social.util.common.PageSettings(startPageSettings);


        com.ibm.bpm.social.util.common.Log.logExit(2);
        return startPageSettings;
      },


      /**
       * write the default start page settings into the user profile
       * 
       * @param {com.ibm.bpm.social.util.common.PageSettings} PageSettings if given, the settings to be
       *        written to the profile
       * 
       * @type {void}
       */
      _writeStartPageToUserProfile: function _writeStartPageToUserProfile_PageHandler(startPageSettings){
        com.ibm.bpm.social.util.common.Log.logEntry(2, startPageSettings.getCurrentProfileString);
        if (startPageSettings) {
          com.ibm.bpm.socialportal.GlobalAssets.setUserPreference(com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME,
            startPageSettings.getCurrentProfileString());
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * 
       * @type {void}
       */
      setStartPage: function setStartPage_PageHandler(startPageSettings, startPageButton){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        var newPageSettings = startPageSettings;

        if (startPageButton.isSelected === true) {
          // we are already on the start page -> User wants to reset the start page to the default start page
          newPageSettings = new com.ibm.bpm.social.util.common.PageSettings();
        }

        this._writeStartPageToUserProfile(newPageSettings);
        this._storedStartPageSettings = newPageSettings;

        this._currentPageSettings = new com.ibm.bpm.social.util.common.PageSettings(startPageSettings);
        this.checkAndUpdateStartPageIcon(this._currentPageSettings, startPageButton);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Compare the new content of the start page against the stored one and update the page icon if the
       * content changed
       * 
       * @type {void}
       */
      checkAndUpdateStartPageIcon: function checkAndUpdateStartPageIcon_PageHandler(startPageSettings, startPageButton /* domNode */){
        com.ibm.bpm.social.util.common.Log.logEntry(2, "", "PageHandler");

        if (startPageSettings.getCurrentProfileString() == this._storedStartPageSettings.getCurrentProfileString()) {
          // set the check mark icon
          dojo.removeClass(startPageButton, "startPageBookmark-unselected");
          dojo.addClass(startPageButton, "startPageBookmark-selected");
          startPageButton.isSelected = true;
          if (startPageButton.firstChild) startPageButton.firstChild.innerHTML = "&#x2713;"; // a11y check mark sign
          if (this._storedStartPageSettings.getCurrentProfileString() == startPageSettings.defaultPageString) {
            // no reset to the default page since we are already there   
            startPageButton.title = this._nlsResources["page_handler_is_default_page"];
          }
          else {
            startPageButton.title = this._nlsResources["page_handler_is_default_page_with_reset"];
          }
        }
        else {
          // set the plus icon
          dojo.removeClass(startPageButton, "startPageBookmark-selected");
          dojo.addClass(startPageButton, "startPageBookmark-unselected");
          startPageButton.isSelected = false;
          if (startPageButton.firstChild) startPageButton.firstChild.innerHTML = "+"; //a11y
          startPageButton.title = this._nlsResources["page_handler_set_start_page_title"];
        }

        com.ibm.bpm.social.util.common.Log.logExit(2, "", "PageHandler");
        return;
      },


      /**
       * Return a div formatted as a start page button
       * 
       * @type {void}
       */
      createStartPageIcon: function createStartPageIcon_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // create the start page icon/button
        var startPageButton = document.createElement("div");

        startPageButton.tabIndex = "0";
        startPageButton.isSelected = true;
        dojo.addClass(startPageButton, "startPageBookmark-selected");

        var a11yNode = document.createElement("span"); //<span class="bpm-social-a11y">+</span>
        a11yNode.className = "bpm-social-a11y";
        a11yNode.innerHtml = "&#x2713;"; // a11y check mark sign 
        startPageButton.appendChild(a11yNode);

        dijit.setWaiRole(startPageButton, "button");
        dijit.setWaiState(startPageButton, "label", this._nlsResources["page_handler_set_start_page"]);

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return startPageButton;
      },


      /**
       * Initialize the dropdown on the dashboard tab
       * 
       * @type {void}
       */
      createDashboardDropdown: function createDashboardDropdown_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (!this.dashboardDropdown) {
          var div = document.createElement("div");
          dojo.byId("processPortalDashboardTabDropDownId").appendChild(div);
          this.dashboardDropdown = new com.ibm.bpm.social.portal.DashboardDropdown({}, div);
          this.dashboardDropdown.dropdownOpen = true;
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * This functions captures a click on the banner to check if the drop down needs to be closed. (because
       * onblur handler of the DashboardDropdown doesn't work here (for any reason)
       * 
       * @param event
       * @type void
       */

      hideDashboardDropdown: function hideDashboardDropdown_PageHandler(event){
        com.ibm.bpm.social.util.common.Log.logEntry(3);
        var target = null;
        if (dojo.isIE == 8) {
          target = event.srcElement;
        }
        else {
          target = event.target;
        }
        if (target.id != "processPortalDashboardsLink" && target.id != "processPortalDashboardTabDropDownImg") {
          if (this.dashboardDropdown && this.dashboardDropdown.dropdownOpen) {
            com.ibm.bpm.social.util.common.Log.log(2, "Catched click on banner and dropdown is open. Hiding ...");
            this.dashboardDropdown.hide();
          }
        }
        com.ibm.bpm.social.util.common.Log.logExit(3);
        return false;
      },

      /**
       * handle a click on the dashboards dropdown arrow
       * 
       * @type {void}
       */
      onDashboardDropdownClick: function onDashboardDropdownClick_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        if (!this.dashboardDropdown) {
          // creating the drop down also opens it
          this.createDashboardDropdown();
        }
        else if (this.dashboardDropdown.dropdownOpen) {
          //DashboardDropdown is already open, close it);
          this.dashboardDropdown.hide();
        }
        else {
          this.dashboardDropdown.show();
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return false;
      },


      /**
       * Event handler for the onDashboardRestart event, delegate this to the dashboard controller
       * 
       * @type {void}
       */
      restartDashboard: function restartDashboard_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
          runURL: "RESTART_DASHBOARD",
          display: "RESTART_DASHBOARD"
        }]);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },

	  checkUrlHash: function() {
		var rc = false;
  		var hash = dojo.hash();
		if (!hash) {
			hash = dojo.cookie("com_ibm_bpm_process_portal_hash");
		}
		dojo.cookie("com_ibm_bpm_process_portal_hash", null, {path: "/"});
		if (hash) {
			dojo.hash("");
			hash = hash.split('/');
			if ((hash.length == 3) && (hash[1] == "tasks")) {
				var taskId = parseInt(hash[2]);
				if (taskId) {
					var taskData = com.ibm.bpm.social.util.common.Util.getTaskData({
						'taskId': taskId
					});
					if (taskData) {
						if (taskData.errorInfo) {
							// 401 is unathorized, so the task exists and we
							// want to let the default error message show up,
							// otherwise assume the id is bad 
							if (taskData.errorInfo.ioArgs.xhr.status != 401) {
								var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["task_not_available"]);
								taskId = null;
							}
						}
					} else {
						taskId = null;
					}
				}
				if (taskId) {
					var delay = 0;
					// on the iPad, delay showing the task completion
					// view to allow initialization to finish first
					if (navigator.userAgent.indexOf("iPad") > -1) {
						delay = 500;
					} 
					this.showTaskCompletion(taskId,delay);
					rc = true;
				} else {
					// log message for bad task id
					com.ibm.bpm.social.util.common.Log.log(2, "The specified task id is invalid: "+hash[2],"PageHandler");
				} 
			}
			else if ((hash.length == 3) && (hash[1] == "processes")) {
				var piid = parseInt(hash[2]);
				if (piid) {
					var instanceData = com.ibm.bpm.social.util.common.Util.getInstanceData({
						'header': true, 
						'instanceId': piid,
						'skipCache': true
					});
					if (instanceData) {
						if (instanceData.errorInfo) {
							// 401 is unathorized, so the process exists and
							// want to let the default error message show up,
							// otherwise assume the id is bad
							if (instanceData.errorInfo.ioArgs.xhr.status != 401) {
								var nls = dojo.i18n.getLocalization("com.ibm.bpm.social", "bpmTask");
								com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showWarning(nls["process_instance_not_available"]);
								piid = null;
							}
						}
					} else {
						piid = null;
					}
				}
				if (piid) {
					var delay = 0;
					// on the iPad, delay showing the task completion
					// view to allow initialization to finish first
					if (navigator.userAgent.indexOf("iPad") > -1) {
						delay = 500;
					} 
					this.displayInstanceDetails(piid,delay);
					rc = true;
				} else {
					// log message for bad process instance id
					com.ibm.bpm.social.util.common.Log.log(2, "The specified process instance id is invalid: "+hash[2],"PageHandler");
				}
			} else {
				// log message for unsupported fragment
				com.ibm.bpm.social.util.common.Log.log(2, "The specified URL fragment is not supported: #"+hash.join('/'),"PageHandler"); 
			}
		}	
		return rc;
	  },

      /**
       * Read the start page settings from user profile and show the start page
       * 
       * @type {void}
       */
      showStartPage: function showStartPage_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);

        this._workPageWidget = dijit.byId("workPage");
        this._workPageDiv = dojo.byId("workPageDiv");
        this._dashboardPageWidget = dijit.byId("dashboardPage");
        this._dashboardPageDiv = dojo.byId("dashboardPageDiv");

		if (this.checkUrlHash()) {
		  // do nothing since hash has been handled
		} else if (this._storedStartPageSettings.tab === this._storedStartPageSettings.tabPrefixes.DASHBOARDS) {
          // activate the dashboards tab
          if (!this.dashboardDropdown) {
            this.createDashboardDropdown();
            // ensure the drop down is closed
            if (this.dashboardDropdown.dropdownOpen) {
              this.dashboardDropdown.hide();
            }
          }

          var dashboardURL = this._storedStartPageSettings.dashboardURL;

          // Lookup dashboard in exposed items
          var item;
          var found = false;
          for ( var i = 0; i < com.ibm.bpm.social.portal.menuItems.length; i++) {
            item = com.ibm.bpm.social.portal.menuItems[i];
            if (item.runURL.indexOf(dashboardURL) != -1) {
              found = true;
              break;
            }
          }
          if (found) {
            this.selectDashboardsTab(item.runURL, item.display);
          }
          else {
            com.ibm.bpm.social.util.common.Log.log(1, "Lookup of dashboard failed.");
            // show error message, the use default
            com.ibm.bpm.social.widgets.dialogs.ErrorDialog.showInfo(this._nlsResources["dashboard_no_longer_available"],
              dojo.string.substitute(this._nlsResources["dashboard_no_longer_available_details"], [dashboardURL]));
            this.selectWorkTab();
          }
        }
        else {
          // in all other cases switch to work page as default page
          this.selectWorkTab();
        }

        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * displayInstanceDetails: Used by the dashboards page for cross-site scripting purposes. Opens the
       * process details for a given instance.
       * 
       * @param bpdId
       */
      displayInstanceDetails: function displayInstanceDetails_PageHandler(bpdId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        this.selectWorkTab();
        if (delay) {
        	delay = delay + 500;
        } else {
        	delay = 500;
        }
        //setTimeout helps with iframe publish issues
        setTimeout(function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_PROCESS_INSTANCE, [{
            'instanceId': bpdId
          }]);
        }, delay);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * 
       * @type {void}
       */
      showTaskCompletion: function showTaskCompletion_PageHandler(tkId,delay){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._workPageDiv.getAttribute('data-loaded') == "false") {
          this._workPageWidget.set("href", '/' + ppConfig.portalContextRoot + '/jsp/socialPortal/work.jsp');
          this._workPageWidget.startup();
          this._workPageDiv.setAttribute('data-loaded', "true");
        }
        if (delay) {
        	delay = delay + 500;
        } else {
        	delay = 500;
        }
        //setTimeout helps with iframe publish issues
        setTimeout(function(){
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_COMPLETION, [{
            "taskId": tkId
          }]);
        }, delay);
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return;
      },


      /**
       * Open the work tab
       * 
       * @param tab
       */
      selectWorkTab: function selectWorkTab_PageHandler(){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        if (this._workPageDiv.getAttribute('data-loaded') == "false") {
          this._workPageWidget.set("href", '/' + ppConfig.portalContextRoot + '/jsp/socialPortal/work.jsp');
          this._workPageWidget.startup();
          this._workPageDiv.setAttribute('data-loaded', "true");
        }
        var wasRealSwitch = this.setSelectedTab('processPortalWorkTab');
        if (!wasRealSwitch) {
          // Trigger a refresh if the user clicked again on the work tab
          dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_TASK_LIST, [{}]);
        }
        com.ibm.bpm.social.util.common.Log.logExit(2);
        return false;
      },

      /**
       * Open the dashboards tab
       * 
       * @param tab
       */
      selectDashboardsTab: function selectDashboardsTab_PageHandler(dashboardURL, dashboardTitle){
        if (this._dashboardPageDiv.getAttribute('data-loaded') == "false") {
          this._dashboardPageWidget.startup();
          this._dashboardPageDiv.setAttribute('data-loaded', "true");
        }

        dojo.publish(com.ibm.bpm.social.portal.Topics.SHOW_DASHBOARD, [{
          runURL: dashboardURL,
          display: "dashboardTitle"
        }]);

        this.setSelectedTab('processPortalDashboardTab'); // , '/' + ppConfig.portalContextRoot + '/jsp/socialPortal/dashboard.jsp?url=' + escape(dashboardURL) + '&title=' + dashboardTitle);

        return false;
      },


      /**
       * handle a click on the portals primary tabs (Work/Dashboards)
       * 
       * @param id the id of the tab to switch to
       * @return {Boolean} false if previous and next page were the same, true otherwise
       */
      setSelectedTab: function setSelectedTab_PageHandler(id){
        com.ibm.bpm.social.util.common.Log.logEntry(2);
        // Remove previously selected tab
        dojo.query(".processPortalTab").removeClass("processPortalSelectedTab");

        // Select the new tab
        dojo.addClass(id, "processPortalSelectedTab");

        var previousPage = dojo.query(".processPortalSelectedPage")[0];
        dojo.removeClass(previousPage, "processPortalSelectedPage");
        dojo.addClass(previousPage, "processPortalNonSelectedPage");

        var targetPage = dojo.byId(dojo.byId(id).getAttribute('data-page'));

        dojo.removeClass(targetPage, "processPortalNonSelectedPage");
        dojo.addClass(targetPage, "processPortalSelectedPage");

        var realRefresh = (previousPage.id != targetPage.id);
        // Publish page switch event
        if (realRefresh) {
          var target = targetPage.id.replace("Div", "");
          var previous = previousPage.id.replace("Div", "");
          dojo.publish("com.ibm.bpm.socialPortal.PageTabSwitchEvent", [{
            showing: target,
            hidding: previous
          }]);
        }

        com.ibm.bpm.social.util.common.Log.logExit(2, "real refresh: " + realRefresh);
        // indicate that a real page switch was done 
        return (realRefresh);
      }
    });


////////////////////////////////////////
// GLOBALS
////////////////////////////////////////


com.ibm.bpm.social.portal.PageHandler.PREFERENCE_NAME = "Portal Default Page";

/**
 * Singleton object representing the currently stored startPage and providing the icon
 */
com.ibm.bpm.social.portal.PageHandler.getInstance = function getInstance_PageHandler(){
  com.ibm.bpm.social.util.common.Log.logEntry(2, "", "PageHandler", "getInstance");
  var handler;
  if (com.ibm.bpm.social.portal.PageHandler._instance) {
    handler = com.ibm.bpm.social.portal.PageHandler._instance;
  }
  else {
    handler = new com.ibm.bpm.social.portal.PageHandler();
    handler.readFromUserProfile();
    com.ibm.bpm.social.portal.PageHandler._instance = handler;
  }
  com.ibm.bpm.social.util.common.Log.logExit(2, "", "PageHandler", "getInstance");
  return handler;
};
